from edu_py_logger.utils import PropertyBaseModel


class LogConfig(PropertyBaseModel):
    """Logging configuration to be set for the server"""

    logger_name: str
    log_level: str = "DEBUG"
    version = 1
    file_path: str = None
    disable_existing_loggers = False
    formatters = {"ecs": {"()": "ecs_logging.StdlibFormatter"}}

    @property
    def handlers(self):
        return {
            "console": {
                "class": "logging.StreamHandler",
                "stream": "ext://sys.stdout",
                "formatter": "ecs",
            },
            "file": {
                "formatter": "ecs",
                "class": "logging.FileHandler",
                "filename": self.file_path or f"{self.logger_name}.log",
            },
        }

    @property
    def loggers(self):
        if self.file_path:
            loggers = ["console", "file"]
        else:
            loggers = ["console"]
        return {
            self.logger_name: {
                "handlers": loggers,
                "level": "DEBUG",
            }
        }


def get_config(name, path=None):
    return LogConfig(logger_name=name, file_path=path).dict()



from unittest import TestCase

from edu_py_logger.config import get_config


class TestLogger(TestCase):
    def test_get_config_with_file(self):
        expected = {
            "logger_name": "name",
            "log_level": "DEBUG",
            "file_path": "logging_path.log",
            "handlers": {
                "console": {
                    "class": "logging.StreamHandler",
                    "stream": "ext://sys.stdout",
                    "formatter": "ecs",
                },
                "file": {
                    "formatter": "ecs",
                    "class": "logging.FileHandler",
                    "filename": "logging_path.log",
                },
            },
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {"ecs": {"()": "ecs_logging.StdlibFormatter"}},
            "loggers": {
                "name": {"handlers": ["console", "file"], "level": "DEBUG"}
            },
        }

        result = get_config("name", "logging_path.log")
        self.assertEqual(result, expected)

    def test_get_config(self):
        expected = {
            "logger_name": "name",
            "log_level": "DEBUG",
            "file_path": None,
            "handlers": {
                "console": {
                    "class": "logging.StreamHandler",
                    "stream": "ext://sys.stdout",
                    "formatter": "ecs",
                },
                "file": {
                    "formatter": "ecs",
                    "class": "logging.FileHandler",
                    "filename": "name.log",
                },
            },
            "version": 1,
            "disable_existing_loggers": False,
            "formatters": {"ecs": {"()": "ecs_logging.StdlibFormatter"}},
            "loggers": {"name": {"handlers": ["console"], "level": "DEBUG"}},
        }

        result = get_config("name")
        self.assertEqual(result, expected)



import logging

from pydantic import BaseModel


class AppContext(BaseModel):
    env: str
    ipv4: str = ""
    ipv6: str = ""
    service_name: str
    service_version: str


class ActionContext(BaseModel):
    correlation_id: str = ""
    trace_id: str = ""
    root_action: str = ""
    current_action: str = ""
    user_id: str = ""


class LoggerService:
    def __init__(self, **kwargs):
        app_context = AppContext(**kwargs)
        self.environment = app_context.env
        context__dict_ = {**app_context.dict(), **ActionContext().dict()}

        self.logger = logging.LoggerAdapter(
            logging.getLogger(app_context.service_name), context__dict_
        )

    def info(self, message, *args, extra=None, **kwargs):
        request_context = ActionContext(**extra).dict() if extra else {}
        self.logger.extra = {
            **self.logger.extra,
            **request_context,
        }
        print(self.logger.extra)
        self.logger.info(message, *args, **kwargs)

    def debug(self, message, *args, extra=None, **kwargs):
        request_context = ActionContext(**extra).dict() if extra else {}
        self.logger.extra = {
            **self.logger.extra,
            **request_context,
        }
        self.logger.debug(message, *args, **kwargs)

    def warning(self, message, *args, extra=None, **kwargs):
        request_context = ActionContext(**extra).dict() if extra else {}
        self.logger.extra = {
            **self.logger.extra,
            **request_context,
        }
        self.logger.warning(message, *args, **kwargs)

    def error(self, message, *args, extra=None, **kwargs):
        request_context = ActionContext(**extra).dict() if extra else {}
        self.logger.extra = {
            **self.logger.extra,
            **request_context,
        }
        self.logger.error(message, *args, **kwargs)

    def exception(self, message, *args, extra=None, exc_info=True, **kwargs):
        request_context = ActionContext(**extra).dict() if extra else {}
        self.logger.extra = {
            **self.logger.extra,
            **request_context,
        }
        self.logger.exception(message, *args, exc_info=exc_info, **kwargs)

    def critical(self, message, *args, extra=None, **kwargs):
        request_context = ActionContext(**extra).dict() if extra else {}
        self.logger.extra = {
            **self.logger.extra,
            **request_context,
        }
        print(self.logger.extra)
        self.logger.critical(message, *args, **kwargs)



from unittest import TestCase
from uuid import uuid4

from fastapi import Request

from edu_py_logger.utils import get_request_extra_data


class TestExtraDataFromRequest(TestCase):
    def test_extra(self):
        trace_id = uuid4()
        request = Request({"type": "http"})
        request.state.trace_id = trace_id
        expected = {"trace_id": trace_id, "correlation_id": "correlation_id"}

        result = get_request_extra_data(
            request, action_context={"correlation_id": "correlation_id"}
        )

        self.assertEqual(result, expected)



from unittest import TestCase, mock

from edu_py_logger.logger import AppContext, LoggerService

app_context = AppContext(
    env="local",
    ipv4="ipv4",
    ipv6="ipv6",
    service_name="name",
    service_version="1.0",
).dict()


class TestLogger(TestCase):
    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_init(self, logging_module_mock):
        LoggerService(**app_context)

        logging_module_mock.getLogger.assert_called_once_with("name")
        logging_module_mock.LoggerAdapter.assert_called_once_with(
            logging_module_mock.getLogger.return_value,
            {
                "env": "local",
                "ipv4": "ipv4",
                "ipv6": "ipv6",
                "service_name": "name",
                "service_version": "1.0",
                "correlation_id": "",
                "trace_id": "",
                "root_action": "",
                "current_action": "",
                "user_id": "",
            },
        )

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_info(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.info("message")

        logger_mock.info.assert_called_once_with("message")
        assert logger_mock.extra == {}

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_info_with_extra(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.info("message", extra={"trace_id": "trace_id"})

        logger_mock.info.assert_called_once_with("message")
        assert logger_mock.extra == {
            "correlation_id": "",
            "trace_id": "trace_id",
            "root_action": "",
            "current_action": "",
            "user_id": "",
        }

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_debug(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.debug("message")

        logger_mock.debug.assert_called_once_with("message")
        assert logger_mock.extra == {}

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_debug_with_extra(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.debug("message", extra={"trace_id": "trace_id"})

        logger_mock.debug.assert_called_once_with("message")
        assert logger_mock.extra == {
            "correlation_id": "",
            "trace_id": "trace_id",
            "root_action": "",
            "current_action": "",
            "user_id": "",
        }

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_warning(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.warning("message")

        logger_mock.warning.assert_called_once_with("message")
        assert logger_mock.extra == {}

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_warning_with_extra(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.warning("message", extra={"trace_id": "trace_id"})

        logger_mock.warning.assert_called_once_with("message")
        assert logger_mock.extra == {
            "correlation_id": "",
            "trace_id": "trace_id",
            "root_action": "",
            "current_action": "",
            "user_id": "",
        }

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_error(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.error("message")

        logger_mock.error.assert_called_once_with("message")
        assert logger_mock.extra == {}

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_error_with_extra(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.error("message", extra={"trace_id": "trace_id"})

        logger_mock.error.assert_called_once_with("message")
        assert logger_mock.extra == {
            "correlation_id": "",
            "trace_id": "trace_id",
            "root_action": "",
            "current_action": "",
            "user_id": "",
        }

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_exception(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.exception("message")

        logger_mock.exception.assert_called_once_with("message", exc_info=True)
        assert logger_mock.extra == {}

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_exception_with_extra(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.exception("message", extra={"trace_id": "trace_id"})

        logger_mock.exception.assert_called_once_with("message", exc_info=True)
        assert logger_mock.extra == {
            "correlation_id": "",
            "trace_id": "trace_id",
            "root_action": "",
            "current_action": "",
            "user_id": "",
        }

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_critical(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.critical("message")

        logger_mock.critical.assert_called_once_with("message")
        assert logger_mock.extra == {}

    @mock.patch("edu_py_logger.logger.logging")
    def test_logger_critical_with_extra(self, logging_module_mock):
        logger_mock = mock.MagicMock()
        logging_module_mock.LoggerAdapter.return_value = logger_mock

        service = LoggerService(**app_context)
        service.critical("message", extra={"trace_id": "trace_id"})

        logger_mock.critical.assert_called_once_with("message")
        assert logger_mock.extra == {
            "correlation_id": "",
            "trace_id": "trace_id",
            "root_action": "",
            "current_action": "",
            "user_id": "",
        }



import time
from uuid import uuid4

from starlette.middleware.base import BaseHTTPMiddleware


class TraceIdProcessTimeMiddleware(BaseHTTPMiddleware):
    async def dispatch(self, request, call_next):
        start_time = time.time()
        request.state.trace_id = str(uuid4())
        request.state.time_started = start_time

        response = await call_next(request)

        process_time = time.time() - start_time
        response.headers["X-Process-Time"] = str(process_time)
        return response



import setuptools

with open("README.rst", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="edu-py-logger",
    version="0.7",
    keywords="fastapi logging",
    url="https://github.com/BlipIQSciences/edu-py-logger",
    author="Julia F",
    author_email="julia@blipiq.com",
    license="MIT",
    packages=setuptools.find_packages(),
    long_description=long_description,
    install_requires=[
        "fastapi", "ecs-logging"
    ],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.8",
        "Topic :: System :: Logging",
    ],
)



from typing import Dict, Union

from fastapi import Body, Request
from pydantic import BaseModel


def get_request_extra_data(
    request: Request, action_context: Dict = Body(None, alias="actionContext")
):
    return {
        **(action_context or {}),
        "trace_id": request.state.trace_id,
    }


class PropertyBaseModel(BaseModel):
    """
    Workaround for serializing properties with pydantic until
    https://github.com/samuelcolvin/pydantic/issues/935
    is solved
    """

    @classmethod
    def get_properties(cls):
        return [
            prop
            for prop in dir(cls)
            if isinstance(getattr(cls, prop), property)
            and prop not in ("__values__", "fields")
        ]

    def dict(
        self,
        *,
        include: Union["AbstractSetIntStr", "MappingIntStrAny"] = None,
        exclude: Union["AbstractSetIntStr", "MappingIntStrAny"] = None,
        by_alias: bool = False,
        skip_defaults: bool = None,
        exclude_unset: bool = False,
        exclude_defaults: bool = False,
        exclude_none: bool = False,
    ) -> "DictStrAny":
        attribs = super().dict(
            include=include,
            exclude=exclude,
            by_alias=by_alias,
            skip_defaults=skip_defaults,
            exclude_unset=exclude_unset,
            exclude_defaults=exclude_defaults,
            exclude_none=exclude_none,
        )
        props = self.get_properties()
        # Include and exclude properties
        if include:
            props = [prop for prop in props if prop in include]
        if exclude:
            props = [prop for prop in props if prop not in exclude]

        # Update the attribute dict with the properties
        if props:
            attribs.update({prop: getattr(self, prop) for prop in props})

        return attribs
